# -*- coding: utf-8 -*-

from flask import Flask, Response, request

from flask_paginated_response import PaginatedResponse

app = Flask('flask-test-response')


@app.route('/healthcheck')
def healthcheck():
    return Response(status=200)


@app.route('/paginated/has_pagination')
def paginated():
    page = int(request.args.get('page', '1'))
    return PaginatedResponse(20, page, 50, Response(status=200))



# -*- coding: utf-8 -*-

from unittest import TestCase

from tests import app

REL_NEXT = 'rel="next"'
REL_PREVIOUS = 'rel="prev"'
REL_FIRST = 'rel="first"'
REL_LAST = 'rel="last"'


class TestPaginatedResponse(TestCase):

    def setUp(self):
        self.test_client = app.test_client()

    def test_should_return_links_headers_next_and_last_when_has_three_pages_and_request_is_first_page(self):
        expected_refs = [REL_LAST, REL_NEXT]
        response = self.test_client.get('paginated/has_pagination')
        link_headers = response.headers.get('Link').split(',')
        self.assertEqual(len(expected_refs), len(link_headers))
        for link_header in link_headers:
            rel_ref = link_header.split('; ')[1]
            self.assertIn(rel_ref, expected_refs)

    def test_should_return_all_link_headers_when_has_has_three_pages_and_request_is_second_page(self):
        expected_refs = [REL_LAST, REL_NEXT, REL_FIRST, REL_PREVIOUS]
        response = self.test_client.get('paginated/has_pagination?page=2')
        link_headers = response.headers.get('Link').split(',')
        self.assertEqual(len(expected_refs), len(link_headers))
        for link_header in link_headers:
            rel_ref = link_header.split('; ')[1]
            self.assertIn(rel_ref, expected_refs)

    def test_should_return_link_headers_when_has_has_three_pages_and_request_is_third_page(self):
        expected_refs = [REL_FIRST, REL_PREVIOUS]
        response = self.test_client.get('paginated/has_pagination?page=3')
        link_headers = response.headers.get('Link').split(',')
        self.assertEqual(len(expected_refs), len(link_headers))
        for link_header in link_headers:
            rel_ref = link_header.split('; ')[1]
            self.assertIn(rel_ref, expected_refs)



# -*- coding: utf-8 -*-

from urllib import urlencode
from urlparse import urlparse, parse_qs, urlunparse

from flask import Response, request


class PaginatedResponse(Response):

    LINK_TEMPLATE = '<{0}size={1}&page={2}>; rel="{3}"'

    def __init__(self, per_page, current_page, total, response, mimetype=None):
        headers = dict()

        link_header = self.__get_link_header(per_page, current_page, total)
        headers.update(link_header)

        headers['X-Total-Count'] = total

        super(PaginatedResponse, self).__init__(
            response=response,
            headers=headers,
            mimetype=mimetype
        )

    def __get_link_header(self, per_page, current_page, total):
        links = list()

        url = PaginatedResponse.__get_url_without_pagination()

        if not PaginatedResponse.__is_first_page(current_page):
            links.append(self.LINK_TEMPLATE.format(url, per_page, 1, 'first'))

        if not PaginatedResponse.__is_last_page(per_page, current_page, total):
            links.append(self.LINK_TEMPLATE.format(url, per_page, (total / per_page) + 1, 'last'))

        if PaginatedResponse.__has_next(per_page, current_page, total):
            links.append(self.LINK_TEMPLATE.format(url, per_page, current_page + 1, 'next'))

        if PaginatedResponse.__has_previous(current_page):
            previous_page = min(current_page - 1, total / per_page)
            links.append(self.LINK_TEMPLATE.format(url, per_page, previous_page, 'prev'))

        return dict(Link=', '.join(links))

    @staticmethod
    def __get_url_without_pagination():
        url = request.url
        parsed_url = urlparse(url)
        query = parse_qs(parsed_url.query)
        query.pop('size', None)
        query.pop('page', None)
        connector = '&' if query else '?'
        parsed_url = parsed_url._replace(query=urlencode(query, True))
        return '{0}{1}'.format(urlunparse(parsed_url), connector)

    @staticmethod
    def __is_first_page(current_page):
        return current_page == 1

    @staticmethod
    def __is_last_page(per_page, current_page, total):
        return per_page * current_page >= total > per_page * (current_page - 1)

    @staticmethod
    def __has_next(per_page, current_page, total):
        return current_page * per_page < total

    @staticmethod
    def __has_previous(current_page):
        return current_page > 1
